@@ -3,13 +3,11 @@ package Net::OAI::Harvester;
 use strict;
 use warnings;
 
-use constant XMLNS_OAI => "http://www.openarchives.org/OAI/2.0/";
-
 use URI;
 use LWP::UserAgent;
 use XML::SAX qw( Namespaces Validation );
 use File::Temp qw( tempfile );
-use Carp qw( carp croak );
+use Carp qw( croak );
 
 use Net::OAI::Error;
 use Net::OAI::ResumptionToken;
@@ -18,14 +16,13 @@ use Net::OAI::ListMetadataFormats;
 use Net::OAI::ListIdentifiers;
 use Net::OAI::ListRecords;
 use Net::OAI::GetRecord;
+use Net::OAI::ListRecords;
 use Net::OAI::ListSets;
 use Net::OAI::Record::Header;
 use Net::OAI::Record::OAI_DC;
 
-our $VERSION = "1.16_12";
+our $VERSION = '1.1';
 our $DEBUG = 0;
-# compatibility mode for metadataHandler
-our $OLDmetadataHandler = 0;
 
 =head1 NAME
 
@@ -115,14 +112,12 @@ iterated over one at a time. The benefit of this is a lower memory footprint
 when (for example) a ListRecords verb is exercised on a repository that 
 returns 100,000 records.
 
-
 =item 2
 
 XML::SAX filters are used which will allow interested developers to write 
 their own metadata parsing packages, and drop them into place. This is useful
 because OAI-PMH is itself metadata schema agnostic, so you can use OAI-PMH 
-to distribute all kinds of metadata (Dublin Core, MARC, EAD, or your favorite
-metadata schema). OAI-PMH does require that a repository at least provides 
+to distribute all kinds of metadata (Dublin Core, MARC, EAD, or your favorite metadata schema). OAI-PMH does require that a repository at least provides 
 Dublin Core metadata as a baseline. Net::OAI::Harvester has built in support for 
 unqualified Dublin Core, and has a framework for dropping in your own parser 
 for other kinds of metadata. If you create a XML::Handler that you would like 
@@ -221,24 +216,25 @@ intersted in. For example:
     print "admin email(s): ", join( ", ", $identity->adminEmail() ), "\n";
     ...
 
-For more details see the L<Net::OAI::Identify> documentation.
+For more details see the Net::OAI::Identify documentation.
 
 =cut 
 
 sub identify {
     my $self = shift;
-    my $uri = $self->{ baseURL }->clone();
+    my $uri = $self->{ baseURL };
     $uri->query_form( 'verb' => 'Identify' );
 
     my $identity = Net::OAI::Identify->new( $self->_get( $uri ) );
-    return $identity if $identity->{ error };
+    return $identity if $identity->errorCode();
 
-    my $error = Net::OAI::Error->new( Handler => $identity );
+    my $token = Net::OAI::ResumptionToken->new( Handler => $identity );
+    my $error = Net::OAI::Error->new( Handler => $token );
     my $parser = _parser( $error ); 
     debug( "parsing Identify response " .  $identity->file() );
     eval { $parser->parse_uri( $identity->file() ) };
-    if ( $@ ) { _xmlError( $error ); } 
-    $error->set_handler( undef );
+    if ( $@ ) {_xmlError( $error ); } 
+    $identity->{ token } = $token->token() ? $token : undef;
     $identity->{ error } = $error;
     return( $identity );
 }
@@ -260,27 +256,29 @@ a particular resource identifier then you can pass in that identifier.
     print "record identifier 1234567 can be retrieved as ",
 	join( ',', $list->prefixes() ),"\n";
 
-See documentation for L<Net::OAI::ListMetadataFormats> for more details.
+See documentation for Net::OAI::ListMetadataFormats for more details.
 
 =cut
 
 sub listMetadataFormats {
     my ( $self, %opts ) = @_;
-    my $uri = $self->{ baseURL }->clone();
-
-    $uri->query_form( verb => 'ListMetadataFormats', 
-        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( identifier )
-      );
+    my $uri = $self->{ baseURL };
+    my %pairs = ( verb => 'ListMetadataFormats' );
+    if ( $opts{ identifier } ) { 
+	$pairs{ identifier } = $opts{ identifier }; 
+    }
+    $uri->query_form( %pairs );
 
     my $list = Net::OAI::ListMetadataFormats->new( $self->_get( $uri ) );
-    return $list if $list->{ error };
+    return $list if $list->errorCode();
 
-    my $error = Net::OAI::Error->new( Handler => $list );
+    my $token = Net::OAI::ResumptionToken->new( Handler => $list );
+    my $error = Net::OAI::Error->new( Handler => $token );
     my $parser = _parser( $error );
     debug( "parsing ListMetadataFormats response: ".$list->file() );
-    eval { $parser->parse_uri( $list->file() ) };
+    eval{ $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
-    $error->set_handler( undef );
+    $list->{ token } = $token->token() ? $token : undef;
     $list->{ error } = $error;
     return( $list );
 }
@@ -291,75 +289,66 @@ getRecord() is used to retrieve a single record from a repository. You must pass
 in the C<identifier> and an optional C<metadataPrefix> parameters to identify 
 the record, and the flavor of metadata you would like. Net::OAI::Harvester 
 includes a parser for OAI DublinCore, so if you do not specifiy a 
-metadataPrefix 'oai_dc' will be assumed. If you would like to drop in your own 
-XML::Handler for another type of metadata use either the C<metadataHandler>
-or the C<recordHandler> parameter, either the name of the class as string
-or an already instantiated object of that class.
+metadataPrefix 'oai_dc' will be assumed. If you would like to drop in you own 
+XML::Handler for another type of metadata use the C<metadataHandler> parameter.
 
-    my $result = $harvester->getRecord( 
+    my $record = $harvester->getRecord( 
 	identifier	=> 'abc123',
     );
 
-    ## did something go wrong?
-    if ( my $oops = $result->errorCode() ) { ... };
-
-    ## get the result as Net::OAI::Record object
-    my $record = $result->record();     # undef if error
-
-    ## directly get the Net::OAI::Record::Header object
-    my $header = $result->header();     # undef if error
-    ## same as
-    my $header = $result->record()->header();     # undef if error
+    ## get the Net::OAI::Record::Header object
+    my $header = $record->header();
 
     ## get the metadata object 
-    my $metadata = $result->metadata(); # undef if error or harvested with recordHandler
+    my $metadata = $record->metadata();
 
     ## or if you would rather use your own XML::Handler 
     ## pass in the package name for the object you would like to create
-    my $result = $harvester->getRecord(
+    my $record = $harvester->getRecord(
 	identifier		=> 'abc123',
-	metadataHandler		=> 'MyHandler'
+	metadataHandler		=> 'MyHandler'	    
     );
-    my $metadata = $result->metadata();
-    
-    my $result = $harvester->getRecord(
-	identifier		=> 'abc123',
-	recordHandler		=> 'MyCompleteHandler'
-    );
-    my $complete_record = $result->recorddata(); # undef if error or harvested with metadataHandler
+    my $metadata = $record->metadata();
     
 =cut 
 
 sub getRecord {
     my ( $self, %opts ) = @_;
 
-    croak( "getRecord(): the 'identifier' parameter is required" )
-	unless defined $opts{ 'identifier' };
-    croak( "getRecord(): the 'metadataPrefix' parameter is required" )
-	unless exists $opts{ 'metadataPrefix' };
-    croak( "getRecord(): recordHandler and metadataHandler are mutually exclusive" )
-        if $opts{ recordHandler } and $opts{ metadataHandler };
+    croak( "you must pass the identifier parameter to getRecord()" )
+	if ( ! exists( $opts{ 'identifier' } ) );
+    croak( "you must pass the metadataPrefix parameter to getRecord()" )
+	if ( ! exists( $opts{ 'metadataPrefix' } ) );
 
-    my $uri = $self->{ baseURL }->clone();
+    my $metadataHandler;
+    if ( exists( $opts{ metadataHandler } ) ) {  
+	my $package = $opts{ metadataHandler };
+	_verifyMetadataHandler( $package );
+	$metadataHandler = $package->new();
+    } else {
+	$metadataHandler = Net::OAI::Record::OAI_DC->new();
+    }
 
-    $uri->query_form( verb => 'GetRecord', 
-        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( identifier metadataPrefix )
-      );
+    my $uri = $self->{ baseURL };
+    $uri->query_form(
+	verb		=> 'GetRecord',
+	identifier	=> $opts{ 'identifier' },
+	metadataPrefix	=> $opts{ 'metadataPrefix' }
+    );
 
-    my $record = Net::OAI::GetRecord->new( $self->_get( $uri ), 
-	recordHandler => $opts{ recordHandler },
-	metadataHandler => $opts{ metadataHandler },
-        );
-    return $record if $record->{ error };
+    my $record = Net::OAI::GetRecord->new( $self->_get( $uri ) );
+    return $record if $record->errorCode();
 
-    my $error = Net::OAI::Error->new( Handler => $record );
+    my $header = Net::OAI::Record::Header->new( Handler => $metadataHandler );
+    my $error = Net::OAI::Error->new( Handler => $header );
     my $parser = _parser( $error ); 
     debug( "parsing GetRecord response " . $record->file() );
-    eval { $parser->parse_uri( $record->file() ) };
+    $parser->parse_uri( $record->file() );
     if ( $@ ) { _xmlError( $error ); } 
 
-    $error->set_handler( undef );
     $record->{ error } = $error;
+    $record->{ metadata } = $metadataHandler;
+    $record->{ header } = $header;
     return( $record );
 
 }
@@ -385,20 +374,7 @@ OAI-PMH spec.
     }
 
 If you would like to use your own metadata handler then you can specify 
-the package name of the handler as the C<metadataHandler> (will be exposed
-to events below the C<metadata> element) or C<recordHandler> (will be
-exposed to the C<record> element and all its children) parameter, passing
-either
-
-=over 4
-
-=item the name of the class as string, in that case a new instance
-will be created for any OAI record encountered or
-
-=item an already instantiated object of that class which will 
-be reused for all records. 
-
-=back
+the package name of the handler. 
 
     my $records = $harvester->listRecords(
 	metadataPrefix	=> 'mods',
@@ -410,26 +386,20 @@ be reused for all records.
 	# $metadata will be a MODS::Handler object
     }
 
-If you want to automatically handle resumption tokens you can achieve
-this with the listAllRecords() method. In this case the C<next()> 
-method transparently causes the next response to be fetched from
-the repository if the current response ran out of records and 
-contained a resumptionToken.
+If you want to automatically handle resumption tokens you can with 
+the listAllRecords() method.
 
 If you prefer you can handle resumption tokens yourself with a 
 loop, and the resumptionToken() method. You might want to do this
 if you are working with a repository that wants you to wait between
-requests or if connectivity problems become an issue during particulary
-long harvesting runs and you want to implement a retransmission
-strategy for failing requests.
+requests.
 
     my $records = $harvester->listRecords( metadataPrefix => 'oai_dc' );
-    my $responseDate = $records->responseDate();
     my $finished = 0;
 
     while ( ! $finished ) {
 
-	while ( my $record = $records->next() ) { # a Net::OAI::Record object
+	while ( my $record = $records->next() ) {
 	    my $metadata = $record->metadata();
 	    # do interesting stuff here 
 	}
@@ -445,37 +415,29 @@ strategy for failing requests.
 
     }
 
-Please note: Since C<listRecords()> stashes away the individual
-records it encounters with C<Storable>, special care has to 
-be taken if the handlers you provided make use of XS modules
-since these objects cannot be reliably handled. Therefore you will
-have to provide the special serializing and deserializing methods
-C<STORABLE_freeze()> and C<STORABLE_thaw()> for the objects
-used by your filter(s).
-
-
 =cut
 
 sub listRecords {
     my ( $self, %opts ) = @_;
 
-    croak( "listRecords(): the 'metadataPrefix' parameter is required" )
-	unless ( exists $opts{ 'metadataPrefix' } 
-              or defined $opts{ 'resumptionToken' } );
-    croak( "listRecords(): recordHandler and metadataHandler are mutually exclusive" )
-        if $opts{ recordHandler } and $opts{ metadataHandler };
+    croak( "you must pass the metadataPrefix parameter to listRecords()" )
+	if ( ! exists( $opts{ 'metadataPrefix' } )
+	    and ! exists( $opts{ 'resumptionToken' } ) );
+    my %pairs = ( 
+	verb		  => 'ListRecords', 
+    );
 
-    my $uri = $self->{ baseURL }->clone();
-
-    $uri->query_form( verb => 'ListRecords', 
-        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( metadataPrefix from until set resumptionToken )
-      );
+    foreach ( qw( metadataPrefix from until set resumptionToken ) ) {
+	if ( exists( $opts{ $_ } ) ) {
+	    $pairs{ $_ } = $opts{ $_ };
+	}
+    }
+    my $uri = $self->{ baseURL };
+    $uri->query_form( %pairs );
 
     my $list = Net::OAI::ListRecords->new( $self->_get( $uri ), 
-	metadataHandler => $opts{ metadataHandler },
-	recordHandler => $opts{ recordHandler },
-        );
-    return $list if $list->{ error };
+	metadataHandler => $opts{ metadataHandler } );
+    return $list if $list->errorCode();
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
@@ -484,27 +446,15 @@ sub listRecords {
     eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
 
-    $token->set_handler( undef );
-    $list->{ token } = $token->token() ? $token : undef;
-
-    $error->set_handler( undef );
     $list->{ error } = $error;
-
+    $list->{ token } = $token->token() ? $token : undef;
     return( $list );
 }
 
 =head2 listAllRecords() 
 
-Does exactly what listRecords() does except the C<next()> method 
-will automatically submit resumption tokens as needed.
-
-    my $records = $harvester->listAllRecords( metadataPrefix => 'oai_dc' );
-
-    while ( my $record = $records->next() ) { # a Net::OAI::Record object until undef
-	my $metadata = $record->metadata();
-	# do interesting stuff here 
-    }
-
+Does exactly what listRecords() does except it will automatically
+submit resumption tokens as needed.
 
 =cut
 
@@ -521,18 +471,18 @@ sub listAllRecords {
 listIdentifiers() takes the same parameters that listRecords() takes, but it 
 returns only the record headers, allowing you to quickly retrieve all the 
 record identifiers for a particular repository. The object returned is a 
-L<Net::OAI::ListIdentifiers> object.
+Net::OAI::ListIdentifiers object.
 
     my $headers = $harvester->listIdentifiers( 
 	metadataPrefix	=> 'oai_dc'
     );
 
     ## iterate through the results with next()
-    while ( my $header = $identifiers->next() ) {  # a Net::OAI::Record::Header object
+    while ( my $header = $identifiers->next() ) { 
 	print "identifier: ", $header->identifier(), "\n";
     }
 
-If you want to automatically handle resumption tokens use listAllIdentifiers().
+If you want to automtically handle resumption tokens use listAllIdentifiers().
 If you are working with a repository that encourages pauses between requests
 you can handle the tokens yourself using the technique described above
 in listRecords().
@@ -541,17 +491,22 @@ in listRecords().
 
 sub listIdentifiers {
     my ( $self, %opts ) = @_;
-    croak( "listIdentifiers(): the 'metadataPrefix' parameter is required" )
-	unless ( exists $opts{ 'metadataPrefix' } 
-              or defined $opts{ 'resumptionToken' } );
-    my $uri = $self->{ baseURL }->clone();
-
-    $uri->query_form( verb => 'ListIdentifiers', 
-        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( metadataPrefix from until set resumptionToken )
-      );
+    croak( "listIdentifiers(): metadataPrefix is a required parameter" ) 
+	if ( ! exists( $opts{ metadataPrefix } )
+	    and ! exists( $opts{ resumptionToken } ) );
+    my $uri = $self->{ baseURL };
+    my %pairs = (
+	verb		=> 'ListIdentifiers', 
+    );
+    foreach ( qw( metadataPrefix from until set resumptionToken ) ) {
+	if ( exists( $opts{ $_ } ) ) {
+	    $pairs{ $_ } = $opts{ $_ };
+	}
+    }
+    $uri->query_form( %pairs );
 
     my $list = Net::OAI::ListIdentifiers->new( $self->_get( $uri ) );
-    return( $list ) if $list->{ error };
+    return( $list ) if $list->errorCode();
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
@@ -559,19 +514,14 @@ sub listIdentifiers {
     debug( "parsing ListIdentifiers response " . $list->file() );
     eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
-
-    $token->set_handler( undef );
     $list->{ token } = $token->token() ? $token : undef; 
-
-    $error->set_handler( undef );
     $list->{ error } = $error;
-
     return( $list );
 }
 
 =head2 listAllIdentifiers()
 
-Does exactly what listIdentifiers() does except C<next()> will automatically 
+Does exactly what listIdentifiers() does except it will automatically 
 submit resumption tokens as needed.
 
 =cut
@@ -584,7 +534,6 @@ sub listAllIdentifiers {
     return( $list );
 }
 
-
 =head2 listSets()
 
 listSets() takes an optional C<resumptionToken> parameter, and returns a 
@@ -601,33 +550,30 @@ spec and the Net::OAI::ListSets docs.
 
 sub listSets {
     my ( $self, %opts ) = @_;
-
-    my $uri = $self->{ baseURL }->clone();
-
-    $uri->query_form( verb => 'ListSets', 
-        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( resumptionToken )
-      );
+    my %pairs = ( verb => 'ListSets' );
+    if ( exists( $opts{ resumptionToken } ) ) {
+	$pairs{ resumptionToken } = $opts{ resumptionToken };
+    }
+    my $uri = $self->{ baseURL };
+    $uri->query_form( %pairs );
 
     my $list = Net::OAI::ListSets->new( $self->_get( $uri ) );
-    return( $list ) if $list->{ error };
+    return( $list ) if $list->errorCode();
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
     my $parser = _parser( $error );
     debug( "parsing ListSets response " . $list->file() );
-    eval { $parser->parse_uri( $list->file() ) };
+    eval{ $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
-
-    $token->set_handler( undef );
-    $list->{ token } = $token->token() ? $token : undef;
-    $error->set_handler( undef );
     $list->{ error } = $error;
+    $list->{ token } = $token->token() ? $token : undef;
     return( $list );
 }
 
 =head2 baseURL()
 
-Gets or sets the base URL for the repository being harvested (as L<URI/>).
+Gets or sets the base URL for the repository being harvested.
 
     $harvester->baseURL( 'http://memory.loc.gov/cgi-bin/oai2_0' );
 
@@ -640,13 +586,7 @@ Or if you want to know what the current baseURL is
 sub baseURL {
     my ( $self, $url ) = @_;
     if ( $url ) { $self->{ baseURL } = URI->new( $url ); } 
-## The HTTP UserAgent modifies its URI object upon execution,
-## therefore we'll always provide a clone <s>have to reconstruct: trim the query part ...</s>
-#    my $c = $self->{ baseURL };           # ->canonical();
-#    if ( $c && ($c =~ /^([^\?]*)\?/) ) {  # $c might be undefined
-#        return $1};
-#    return $c;
-    return $self->{ baseURL };
+    return( $self->{ baseURL }->as_string() );
 }
 
 =head2 userAgent()
@@ -660,7 +600,8 @@ agent string, timeout, or some other feature.
 sub userAgent {
     my ( $self, $ua ) = @_;
     if ( $ua ) { 
-	$ua->isa('LWP::UserAgent') or croak( "userAgent() needs a valid LWP::UserAgent" );
+	croak( "userAgent() needs a valid LWP::UserAgent" ) 
+	    if ref( $ua ) ne 'LWP::UserAgent';
 	$self->{ userAgent } = $ua;
     }
     return( $self->{ userAgent } );
@@ -673,54 +614,34 @@ sub _get {
     my $ua = $self->{ userAgent };
 
     my ($fh, $file);
+
     if ( $self->{ dumpDir } ) {
         my $filePrefix = $self->{lastDump}++;
         $file = sprintf("%s/%08d.xml", $self->{dumpDir}, $filePrefix);
         $fh = IO::File->new($file, 'w');
-    } else {
-        ( $fh, $file ) = tempfile(UNLINK => 1);
+    }
+
+    else {
+        ( $fh, $file ) = tempfile();
     }
 
     debug( "fetching ".$uri->as_string() );
     debug( "writing to file: $file" );
     my $request = HTTP::Request->new( GET => $uri->as_string() );
-    my $response = $ua->request( $request, sub { print $fh shift; }, 8192 );
+    my $response = $ua->request( $request, sub { print $fh shift; }, 4096 );
     close( $fh );
 
     if ( $response->is_error() ) { 
-# HTTP::Request does not provide a file in case of HTTP level errors,
-# therefore we do not return the name of the non-existant file but
-# rather the original HTTP::Response object
         debug( "caught HTTP level error" . $response->message() );
         my $error = Net::OAI::Error->new(
             errorString     => 'HTTP Level Error: ' . $response->message(),
-            errorCode       => $response->code(),
-            HTTPError       => $response,
-            HTTPRetryAfter  => $response->header("Retry-After") || "",
+            errorCode       => $response->code()
         );
 	return( 
-#	    file	    => $file, 
+	    file	    => $file, 
             error           => $error
 	);
     }
-    if ( my $ct = $response->header("Content-Type") ) {
-        debug( "Content-type $ct in HTTP response" );
-        unless ( $ct =~ /^text\/xml(;|$)/ ) {
-            return (error => Net::OAI::Error->new(errorCode => 'xmlContentError',
-                                                errorString => "Content-Type: text/xml is mandatory (got: $ct)!"),
-                                                  HTTPError => $response,
-                                            HTTPRetryAfter  => $response->header("Retry-After") || "",
-                   )
-          };
-        if ( $ct =~ /; charset=(\S+)/ ) {
-            my $cs = $1;
-            return (error => Net::OAI::Error->new(errorCode => 'xmlContentError',
-                                                errorString => "charset=UTF-8 is mandatory (got: $cs)!"),
-                                                  HTTPError => $response,
-                                            HTTPRetryAfter  => $response->header("Retry-After") || "",
-                   ) unless $cs =~ /^utf-8/i;
-          };
-    }
 
     return( 
 	    file	    => $file,
@@ -731,62 +652,39 @@ sub _get {
 sub _parser {
     my $handler = shift;
     my $factory = XML::SAX::ParserFactory->new();
-    my $parser;
-    $factory->require_feature(Namespaces);
-    eval { $parser = $factory->parser( Handler => $handler ) };
-    carp ref($factory)." threw an exception:\n\t$@" if $@;
-
-    if ( $parser && ref($parser) ) {
-        debug( "using SAX parser " . ref($parser) . " " . $parser->VERSION );
-        return $parser;
-      };
-
-    carp "!!! Please check your setup of XML::SAX, especially ParserDetails.ini !!!\n";
-    local($XML::SAX::ParserPackage) = "XML::SAX::PurePerl";
-    eval { $parser = $factory->parser( Handler => $handler ) };
-    carp ref($factory)." threw an exception again:\n\t$@" if $@;
-    if ( $parser && ref($parser) ) {
-        carp "Successfuly forced assignment of a parser: " . ref($parser) . " " . $parser->VERSION ."\n";
-        return $parser;
-      };
-
-    croak( ref($factory)." on request did not even give us the default XML::SAX::PurePerl parser.\nGiving up." );
+    return( $factory->parser( Handler => $handler ) );
 }
 
 sub _xmlError {
     my $e = shift;
-    carp "caught xml parsing error: $@";
+    warn "caught xml parsing error: $@";
     $e->errorString( "XML parsing error: $@" );
     $e->errorCode( 'xmlParseError' );
 }
 
 
-sub _verifyHandler {
-    my $package_or_instance = shift;
-    if ( ref($package_or_instance) ) {
-        $package_or_instance->isa('XML::SAX::Base')
-            or _fatal( "Handler $package_or_instance must inherit from XML::SAX::Base\n" )
-      }
-    else {
-        eval( "use $package_or_instance" );
-        _fatal( "unable to locate Handler $package_or_instance in: " . 
-	    join( "\n\t", @INC ) ) if $@; 
-        _fatal( "Handler $package_or_instance must inherit from XML::SAX::Base\n" )
-            if ( ! grep { 'XML::SAX::Base' } eval( '@' . $package_or_instance . '::ISA' ) );
-      }
+sub _verifyMetadataHandler {
+    my $package = shift;
+    eval( "use $package" );
+    _fatal( "unable to locate metadataHandler $package in: " . 
+	join( "\n\t", @INC ) ) if $@; 
+    _fatal( "metadataHandler $package must inherit from XML::SAX::Base\n" )
+	if ( ! grep { 'XML::SAX::Base' } eval( '@' . $package . '::ISA' ) );
     return( 1 );
 }
 
 
 sub debug {
-    return unless $Net::OAI::Harvester::DEBUG;
     my $msg = shift; 
-    carp "oai-harvester: " . localtime() . ": $msg\n";
+    if ( $Net::OAI::Harvester::DEBUG ) { 
+	print STDERR "oai-harvester: " . localtime() . ": $msg\n";
+    }
 }
 
 sub _fatal {
     my $msg = shift;
-    croak "fatal: $msg";
+    print STDERR "fatal: $msg\n\n";
+    exit( 1 );
 }
 
 =head1 DIAGNOSTICS
@@ -796,9 +694,6 @@ then set Net::OAI::Harvester::DEBUG to a true value.
 
     $Net::OAI::Harvester::DEBUG = 1;
 
-
-
-
 =head1 PERFORMANCE 
 
 XML::SAX is used for parsing, but it presents a generalized interface to many 
@@ -811,14 +706,6 @@ If you have a particular parser you want to use you can set the
 $XML::SAX::ParserPackage variable appropriately. See XML::SAX::ParserFactory
 documentation for details.
 
-
-
-=head1 ENVIRONMENT
-
-The modules use LWP for HTTP operations, thus C<PERL_LWP_ENV_PROXY> controls
-wether the "_proxy" environment settings shall be honored.
-
-
 =head1 TODO
 
 =over 4 
@@ -854,11 +741,6 @@ Create common handlers for other metadata formats (MARC, qualified DC, etc).
 
 =item *
 
-Or at least provide a generic record handler as fallback, since using
-L<Net::OAI::Record::OAI_DC> makes absolutely no sense except for ... oai_dc records.
-
-=item *
-
 Selectively load Net::OAI::* classes as needed, rather than getting all of them 
 at once at the beginning of Net::OAI::Harvester.
 
@@ -914,15 +796,7 @@ L<Net::OAI::Record::Header>
 
 =item *
 
-L<Net::OAI::Record::OAI_DC>
-
-=item *
-
-L<Net::OAI::Record::DocumentHelper>
-
-=item *
-
-L<Net::OAI::Record::NamespaceFilter>
+L<Net::OAI::Record::Metadata>
 
 =item *
 
@@ -934,20 +808,20 @@ L<Storable>
 
 =back
 
-
 =head1 AUTHORS
 
+=over 4
+
+=item * 
+
 Ed Summers <ehs@pobox.com>
 
+=item *
+
 Martin Emmerich <Martin.Emmerich@oew.de>
 
-Thomas Berger <ThB@gymel.com>
-
-=head1 LICENSE
-
-This is free software, you may use it and distribute it under the same terms as Perl itself.
+=back
 
 =cut
 
 1;
-
