@@ -1,7 +1,8 @@
 package Net::OAI::ListSets;
 
 use strict;
-use base qw( XML::SAX::Base Net::OAI::Base );
+use base qw( XML::SAX::Base );
+use base qw( Net::OAI::Base );
 
 =head1 NAME
 
@@ -20,6 +21,7 @@ Net::OAI::ListSets - The results of the ListSets OAI-PMH verb.
 sub new {
     my ( $class, %opts ) = @_;
     my $self = bless \%opts, ref( $class ) || $class;
+    $self->{ insideSet } = 0;
     $self->{ specs } = {};
     return( $self );
 }
@@ -55,19 +57,18 @@ sub setName {
 sub start_element {
     my ( $self, $element ) = @_;
     $self->SUPER::start_element( $element );
-    return unless $element->{ NamespaceURI } eq Net::OAI::Harvester::XMLNS_OAI;
-    push( @{ $self->{ tagStack } }, $element->{ LocalName } );
+    push( @{ $self->{ tagStack } }, $element->{ Name } );
 }
 
 sub end_element {
     my ( $self, $element ) = @_;
-    $self->SUPER::end_element( $element );
-    return unless $element->{ NamespaceURI } eq Net::OAI::Harvester::XMLNS_OAI;
-    pop( @{ $self->{ tagStack } } );
-    if ( $element->{ LocalName } eq 'set' ) { 
+    if ( $element->{ Name } eq 'set' ) { 
 	$self->{ specs }{ $self->{ setSpec } } = $self->{ setName };
-	$self->{ setSpec } = $self->{ setName } = undef;
+	$self->{ setSpec } = undef;
+	$self->{ setName } = undef;
     }
+    $self->SUPER::end_element( $element );
+    pop( @{ $self->{ tagStack } } );
 }
 
 sub characters {
